package api

import (
	"bufio"
	"crypto/sha512"
	"encoding/base32"
	"encoding/json"
	"errors"
	"fmt"
	"log"
	"net/http"
	"os"
	"path"
	"strings"

	"srs.epita.fr/fic-server/libfic"
)

var CloudDAVBase string
var CloudUsername string
var CloudPassword string

type uploadedFile struct {
	URI       string
	Digest    []byte
	Path      string
	Parts     []string
}

func createExerciceFile(theme fic.Theme, exercice fic.Exercice, args map[string]string, body []byte) (interface{}, error) {
	var uf uploadedFile
	if err := json.Unmarshal(body, &uf); err != nil {
		return nil, err
	}

	var hash [sha512.Size]byte
	var logStr string
	var fromURI string
	var getFile func(string) (error)

	if uf.URI != "" {
		hash = sha512.Sum512([]byte(uf.URI))
		logStr = "Import file from Cloud: " + uf.URI + " =>"
		fromURI = uf.URI
		getFile = func(dest string) error { return getCloudFile(uf.URI, dest); }
	} else if uf.Path != "" && len(uf.Parts) > 0 {
		hash = sha512.Sum512([]byte(uf.Path))
		logStr = fmt.Sprintf("Import file from local FS: %s =>", uf.Parts)
		fromURI = uf.Path
		getFile = func(dest string) error {
			if fdto, err := os.Create(dest); err != nil {
				return err
			} else {
				writer := bufio.NewWriter(fdto)
				for _, partname := range uf.Parts {
					if fdfrm, err := os.Open(partname); err != nil {
						return err
					} else {
						reader := bufio.NewReader(fdfrm)
						reader.WriteTo(writer)
						writer.Flush()
						fdfrm.Close()
					}
				}
				fdto.Close()
			}
			return nil
		}
	} else if uf.Path != "" {
		hash = sha512.Sum512([]byte(uf.Path))
		logStr = "Import file from local FS: " + uf.Path + " =>"
		fromURI = uf.Path
		getFile = func(dest string) error { return os.Symlink(uf.Path, dest); }
	} else {
		return nil, errors.New("URI or path not filled")
	}

	pathname := path.Join(fic.FilesDir, strings.ToLower(base32.StdEncoding.EncodeToString(hash[:])), path.Base(fromURI))

	if _, err := os.Stat(pathname); os.IsNotExist(err) {
		log.Println(logStr, pathname)
		if err := os.MkdirAll(path.Dir(pathname), 0777); err != nil {
			return nil, err
		} else if err := getFile(pathname); err != nil {
			return nil, err
		}
	}

	return exercice.ImportFile(pathname, fromURI)
}

func getCloudFile(pathname string, dest string) error {
	client := http.Client{}
	if req, err := http.NewRequest("GET", CloudDAVBase+pathname, nil); err != nil {
		return err
	} else {
		req.SetBasicAuth(CloudUsername, CloudPassword)
		if resp, err := client.Do(req); err != nil {
			return err
		} else {
			defer resp.Body.Close()

			if fd, err := os.Create(dest); err != nil {
				return err
			} else {
				defer fd.Close()

				if resp.StatusCode != http.StatusOK {
					return errors.New(resp.Status)
				} else {
					writer := bufio.NewWriter(fd)
					reader := bufio.NewReader(resp.Body)
					reader.WriteTo(writer)
					writer.Flush()
				}
			}
		}
	}
	return nil
}
